@@ -1,5 +1,4 @@
-/* $Id: mswriteimport.h 218419 2003-04-05 09:32:21Z dang $ */
-
+/* $Id: mswriteimport.h 508787 2006-02-12 18:28:12Z ingwa $ */
 /* This file is part of the KDE project
    Copyright (C) 2001-2003 Clarence Dang <dang@kde.org>
 
@@ -14,14 +13,14 @@
 
    You should have received a copy of the GNU Library General Public License
    Version 2 along with this library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef MSWRITEIMPORT_H
 #define MSWRITEIMPORT_H
 
-#include <koFilter.h>
+#include <KoFilter.h>
 
 class WRIDevice;
 class MSWrite::InternalParser;
