@@ -1,10 +1,8 @@
-//
-
 /*
    This file is part of the KDE project
-   Copuright 2001 Michael Johnson <mikej@xnet.com>
-   Copyright 2001, 2002, 2003, 2004 Nicolas GOUTTE <goutte@kde.org>
-   Copyright 2002 Ariya Hidayat <ariya@kde.org>
+   Copuright (C) 2001 Michael Johnson <mikej@xnet.com>
+   Copyright (C) 2001, 2002, 2003, 2004 Nicolas GOUTTE <goutte@kde.org>
+   Copyright (C) 2002 Ariya Hidayat <ariya@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -18,8 +16,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qstring.h>
@@ -27,8 +25,8 @@
 #include <qfile.h>
 #include <qfileinfo.h>
 #include <qfontinfo.h>
+#include <qfontdatabase.h>
 #include <qpicture.h>
-#include <qregion.h> // for #include <kdebugclasses.h>
 #include <qimage.h>
 #include <qregexp.h>
 #include <qcolor.h>
@@ -37,7 +35,6 @@
 #include <klocale.h>
 #include <kglobal.h>
 #include <kdebug.h>
-#include <kdebugclasses.h>
 
 #include <KWEFUtil.h>
 #include <KWEFBaseWorker.h>
@@ -173,7 +170,8 @@ QString RTFWorker::makeTable(const FrameAnchor& anchor)
         QString endOfParagraph;
         QValueList<ParaData> *paraList = (*itCell).paraList;
         QValueList<ParaData>::ConstIterator it;
-        for (it=paraList->begin();it!=paraList->end();it++)
+		QValueList<ParaData>::ConstIterator end(paraList->end());
+        for (it=paraList->begin();it!=end;++it)
         {
             rowText += endOfParagraph;
             rowText += ProcessParagraphData( (*it).text,(*it).layout,(*it).formattingList);
@@ -376,7 +374,7 @@ QString RTFWorker::ProcessParagraphData ( const QString &paraText,
         markup += "\\intbl";
 
 //lists
-if (layout.counter.style)
+    if (layout.counter.style)
         {
         markup += "{\\pntext\\pard\\plain";
         if( layout.formatData.text.fontSize >= 0)
@@ -395,14 +393,14 @@ if (layout.counter.style)
         if (!layout.counter.lefttext.isEmpty() && layout.counter.lefttext != "{" && layout.counter.lefttext != "}")
         {
                 markup += layout.counter.lefttext;
-        }        
+        }
         switch (layout.counter.style)
         {
         case 6:
         {
                 //custom bullets (one char)
                 //TODO: use correct character/sign for bullet
-                
+
             markup += layout.counter.customCharacter;
             break;
         }
@@ -462,9 +460,9 @@ if (layout.counter.style)
         markup += "\\pnlvl";
         markup += QString::number(11 - layout.counter.style);
         }
-        
+
         switch (layout.counter.style)
-        {        
+        {
         case 1:
         {
         markup += "\\pndec";
@@ -494,13 +492,14 @@ if (layout.counter.style)
         markup += "\\pndec";
         }
         markup += "{\\pntxtb ";
-        markup += layout.counter.lefttext;  
+        markup += layout.counter.lefttext;
         markup += " }";
     }
         markup += "{\\pntxta ";
-        markup += layout.counter.righttext;  
+        markup += layout.counter.righttext;
         markup += " }";
-        
+
+        // ### FIXME: that is too late! And why at every list paragraph? (See bug #88241)
         if (layout.counter.start!=0)
         {
         markup += "\\pnstart";
@@ -511,7 +510,7 @@ if (layout.counter.style)
         markup += "\\pnstart1";
         }
         markup += "\\pnindent0\\pnhang";
-        
+
         if( layout.formatData.text.fontSize > 0 )
         {
         markup += "\\pnfs";
@@ -522,9 +521,9 @@ if (layout.counter.style)
         {
             markup += lookupFont("\\pnf", layout.formatData.text.fontName);
         }
-        
+
         markup += "}";
-}    
+}
 
 
     LayoutData styleLayout;
@@ -569,7 +568,7 @@ if (layout.counter.style)
 	    }
             else if (4==(*paraFormatDataIt).id)
             {
-                // ### TODO: put dtae/time fields into own method.
+                // ### TODO: put date/time fields into own method.
                 if ( (0==(*paraFormatDataIt).variable.m_type) // date
                     || (2==(*paraFormatDataIt).variable.m_type) ) // time
                 {
@@ -604,10 +603,36 @@ if (layout.counter.style)
                             key += ' ';
                             key += KGlobal::locale()->timeFormat();
                         }
-                        kdDebug(30515) << "KDE format:  " << key << endl;
-                        key=QString::null; // ### FIXME: KControl's key seems to differ from KWord :-(
+
+                        kdDebug(30515) << "Locale date in KLocale format:  " << key << endl;
+
+                        // KLocale's key differ from KWord
+
+                        // Date
+                        key.replace( "%Y", "yyyy" );    // Year 4 digits
+                        key.replace( "%y", "yy" );      // Year 2 digits
+                        key.replace( "%n", "M" );       // Month 1 digit
+                        key.replace( "%m", "MM" );      // Month 2 digits
+                        key.replace( "%e", "d" );       // Day 1 digit
+                        key.replace( "%d", "dd" );      // Day 2 digits
+                        key.replace( "%b", "MMM" );     // Month 3 letters
+                        key.replace( "%B", "MMMM" );    // Month all letters
+                        key.replace( "%a", "ddd" );     // Day 3 letters
+                        key.replace( "%A", "dddd" );    // Day all letters
+                        // 12h
+                        key.replace( "%p", "am/pm" );   // AM/PM (KLocale knows it only lower case)
+                        key.replace( "%I", "hh" );      // 12 hour 2 digits
+                        key.replace( "%l", "h" );       // 12 hour 1 digits
+                        // 24h
+                        key.replace( "%H", "HH" );      // 24 hour 2 digits
+                        key.replace( "%k", "H" );       // 24 hour 1 digit
+                        // Other times
+                        key.replace( "%M", "mm" );      // minute 2 digits (KLocale knows it with 2 digits)
+                        key.replace( "%S", "ss" );      // second 2 digits (KLocale knows it with 2 digits)
+
+                        kdDebug(30515) << "Locale date in RTF format:  " << key << endl;
                     }
-                    if (!key.isEmpty())
+                    else if (!key.isEmpty())
                     {
                         const QRegExp regexp("AP",false); // Not case-sensitive
                         if (regexp.search(key)!=-1)
@@ -699,17 +724,19 @@ if (layout.counter.style)
                     {
                         QString fstr;
                         QValueList<ParaData>::ConstIterator it;
-                        for (it=paraList->begin();it!=paraList->end();it++)
+                        QValueList<ParaData>::ConstIterator end(paraList->end());
+                        const QString prefixSaved = m_prefix;
+                        m_prefix = QString::null;
+                        for (it=paraList->begin();it!=end;++it)
                             fstr += ProcessParagraphData( (*it).text, (*it).layout,(*it).formattingList);
+                        m_prefix = prefixSaved;
                         content += "{\\super ";
                         content += automatic ? "\\chftn " : value;
                         content += "{\\footnote ";
                         content += "{\\super ";
                         content += automatic ? "\\chftn " : value;
                         content += fstr;
-                        content += " }";
-                        content += " }";
-                        content += " }";
+                        content += "}}}";
                     }
                 }
                 else
@@ -722,7 +749,7 @@ if (layout.counter.style)
             {
                 kdDebug(30515) << "Found an anchor of type: " << (*paraFormatDataIt).frameAnchor.type << endl;
                 // We have an image, a clipart or a table
-               
+
 		if (6==(*paraFormatDataIt).frameAnchor.type)
                 {
 
@@ -740,13 +767,13 @@ if (layout.counter.style)
 			{
 			    m_prefix = "\\par";
 			}
-		    } 
+		    }
 		    str += makeTable((*paraFormatDataIt).frameAnchor);
 		}
                 else if ((2==(*paraFormatDataIt).frameAnchor.type) || (5==(*paraFormatDataIt).frameAnchor.type))
                 {
                     content += makeImage((*paraFormatDataIt).frameAnchor);
-                
+
                 }
 	    }
         }
@@ -767,7 +794,7 @@ if (layout.counter.style)
 	{
 	   m_prefix = "\\par";
         }
-    } 
+    }
     if (str.isEmpty())
     {
      str ="\\par\\pard\\plain";
@@ -807,7 +834,8 @@ bool RTFWorker::doHeader(const HeaderData& header)
     str += " {";
 
     QValueList<ParaData>::ConstIterator it;
-    for (it=header.para.begin();it!=header.para.end();it++)
+	QValueList<ParaData>::ConstIterator end(header.para.end());
+    for (it=header.para.begin();it!=end;++it)
         content += ProcessParagraphData( (*it).text,(*it).layout,(*it).formattingList);
 
     if (content!="\\par\\pard\\plain")
@@ -816,7 +844,7 @@ bool RTFWorker::doHeader(const HeaderData& header)
     str += "}";
 
     str += "}";
-    
+
     m_textBody += str;
     }
     m_prefix=QString::null;
@@ -841,7 +869,8 @@ bool RTFWorker::doFooter(const FooterData& footer)
     str += " {";
 
     QValueList<ParaData>::ConstIterator it;
-    for (it=footer.para.begin();it!=footer.para.end();it++)
+	QValueList<ParaData>::ConstIterator end(footer.para.end());
+    for (it=footer.para.begin();it!=end;++it)
         content += ProcessParagraphData( (*it).text,(*it).layout,(*it).formattingList);
 
     if (content!="\\par\\pard\\plain")
@@ -850,7 +879,7 @@ bool RTFWorker::doFooter(const FooterData& footer)
     str += "}";
 
     str += "}";
-    
+
     m_textBody += str;
     }
     m_prefix=QString::null;
@@ -875,7 +904,8 @@ bool RTFWorker::doOpenFile(const QString& filenameOut, const QString& /*to*/)
 
     m_streamOut=new QTextStream(m_ioDevice);
 
-    m_streamOut->setEncoding(QTextStream::UnicodeUTF8); // We want ASCII, so use UTF-8
+    // ### TODO: should "CP 1252" be used directly? (But RTFWorker::escapeRtfText is beased on ISO-8859-1 only.)
+    m_streamOut->setEncoding(QTextStream::Latin1); // We are declaring the RTF document as CP 1252, so use ISO-8859-1
 
     m_fileName=filenameOut;
 
@@ -897,7 +927,7 @@ bool RTFWorker::doOpenDocument(void)
     // Make the file header
 
     // Note: we use \\ansicpg1252 because 1200 is not supposed to be supported
-    // Note2: we assume using \\ansicpg1252 in RTFWorker::escapeRtfText
+    // Note2: we assume using \\ansicpg1252 in RTFWorker::escapeRtfText and in RTFWorker::doOpenFile
     *m_streamOut << "{\\rtf1\\ansi\\ansicpg1252\\uc1\\deff0" << m_eol;
 
     // Default color table
@@ -916,13 +946,13 @@ void RTFWorker::writeFontData(void)
     kdDebug(30515) << "Fonts:" << m_fontList << endl;
     *m_streamOut << "{\\fonttbl";
     uint count;
+    QFontDatabase fontDatabase;
     QStringList::ConstIterator it;
     for (count=0, it=m_fontList.begin();
         it!=m_fontList.end();
         count++, it++)
     {
-        QFontInfo info(*it);
-        QString strLower(info.family().lower());
+        const QString strLower( (*it).lower() );
         *m_streamOut << "{\\f" << count;
         if ( (strLower.find("symbol")>-1) || (strLower.find("dingbat")>-1) )
             *m_streamOut << "\\ftech";
@@ -932,13 +962,14 @@ void RTFWorker::writeFontData(void)
 #if 1
         else
         {
-            // We do not know the font type that we have and we cannot even tell if it is TrueType
+            // We do not know the font type that we have
             *m_streamOut << "\\fnil";
         }
 #else
         else
         // QFontInfo:styleHint() does not guess anything, it just returns what is in the QFont. Nothing put in, nothing gets out.
         {
+            QFontInfo info(*it);
             switch (info.styleHint())
             {
             case QFont::SansSerif:
@@ -965,9 +996,9 @@ void RTFWorker::writeFontData(void)
             }
         }
 #endif
-        // ### TODO: check if QFontInfo::fixedPitch is really working
-        *m_streamOut << "\\fprq" << (info.fixedPitch()?1:2) << " "; // font definition
-        *m_streamOut << escapeRtfText(info.family()); // ### TODO: does RTF allows brackets in the font names?
+        // ### TODO: \fcharset would be mandatory but Qt3 does not give us the font charset. :-(
+        *m_streamOut << "\\fprq" << ( fontDatabase.isFixedPitch( *it ) ? 1 : 2 ) << " "; // font definition
+        *m_streamOut << escapeRtfText( *it );
         *m_streamOut <<  ";}" << m_eol; // end font table entry
     }
     *m_streamOut << "}"; // end of font table
@@ -1010,7 +1041,7 @@ void RTFWorker::writeStyleData(void)
         // Find the number of the following style
         uint counter=0;  // counts position in style table starting at 0
         QValueList < LayoutData > ::ConstIterator it2;
-        for( it2 =  m_styleList.begin(); it2 != m_styleList.end(); counter++, it2++ )
+        for( it2 =  m_styleList.begin(); it2 != m_styleList.end(); counter++, ++it2 )
         {
             if ( (*it2).styleName == (*it).styleFollowing )
             {
@@ -1077,6 +1108,19 @@ bool RTFWorker::doFullDocumentInfo(const KWEFDocumentInfo& docInfo)
         m_textDocInfo += "}";
     }
 
+    if ( !docInfo.keywords.isEmpty() )
+    {
+        m_textDocInfo += "{\\keywords ";
+        m_textDocInfo += escapeRtfText( docInfo.keywords );
+        m_textDocInfo += "}";
+    }
+    if ( !docInfo.subject.isEmpty() )
+    {
+        m_textDocInfo += "{\\subject ";
+        m_textDocInfo += escapeRtfText( docInfo.subject );
+        m_textDocInfo += "}";
+    }
+
     if ( !docInfo.company.isEmpty() )
     {
         m_textDocInfo += "{\\company ";
@@ -1085,7 +1129,7 @@ bool RTFWorker::doFullDocumentInfo(const KWEFDocumentInfo& docInfo)
     }
 
     // Now add who we are in a \comment
-    QString revision("$Revision: 2.96.2.2 $");
+    QString revision("$Revision: 541888 $");
     m_textDocInfo += "{\\comment ";
     m_textDocInfo += "Generated by KWord's RTF Export Filter";
     m_textDocInfo += revision.mid(10).remove('$'); // has a leading and a trailing space.
@@ -1499,8 +1543,8 @@ QString RTFWorker::layoutToRtf(const LayoutData& layoutOrigin,
 #endif
 
     if (force
-        || (layoutOrigin.lineSpacingType!=layoutOrigin.lineSpacingType)
-        || (layoutOrigin.lineSpacing!=layoutOrigin.lineSpacing))
+        || ( layoutOrigin.lineSpacingType != layout.lineSpacingType )
+        || ( layoutOrigin.lineSpacing != layout.lineSpacing ) )
     {
         if ( layout.lineSpacingType==LayoutData::LS_SINGLE  )
            ;// do nothing, single linespace is default in RTF
@@ -1522,14 +1566,14 @@ QString RTFWorker::layoutToRtf(const LayoutData& layoutOrigin,
            strLayout += QString("\\sl-%1\\slmult0").arg(int(layout.lineSpacing)*20);
 
         else
-            kdWarning(30515) << "Curious lineSpacingType: " << layout.lineSpacingType << " (Ignoring!)" << endl;
+            kdWarning(30515) << "Unsupported lineSpacingType: " << layout.lineSpacingType << " (Ignoring!)" << endl;
     }
 
     if (!layout.tabulatorList.isEmpty()
         && (force || (layoutOrigin.tabulatorList!=layout.tabulatorList) ))
     {
         TabulatorList::ConstIterator it;
-        for (it=layout.tabulatorList.begin();it!=layout.tabulatorList.end();it++)
+        for (it=layout.tabulatorList.begin();it!=layout.tabulatorList.end();++it)
         {
             switch ((*it).m_type)
             {
@@ -1597,7 +1641,7 @@ QString RTFWorker::lookupFont(const QString& markup, const QString& fontName)
     QStringList::ConstIterator it;
 
     // search font table for this font
-    for( it = m_fontList.begin(); it != m_fontList.end(); counter++, it++ )
+    for( it = m_fontList.begin(); it != m_fontList.end(); counter++, ++it )
     {
         if((*it) == cookedFontName)  // check for match
         {
@@ -1625,7 +1669,7 @@ QString RTFWorker::lookupColor(const QString& markup, const QColor& color)
     QValueList < QColor > ::ConstIterator it;
 
     // search color table for this color
-    for( it =  m_colorList.begin(); it != m_colorList.end(); counter++, it++ )
+    for( it =  m_colorList.begin(); it != m_colorList.end(); counter++, ++it )
     {
         if ( (*it) == color )
         {
@@ -1650,9 +1694,10 @@ QString RTFWorker::lookupStyle(const QString& styleName, LayoutData& returnLayou
     QString strMarkup("\\s");  // Holds RTF markup for the style
 
     QValueList < LayoutData > ::ConstIterator it;
+	QValueList < LayoutData > ::ConstIterator end(m_styleList.end());
 
     // search color table for this color
-    for( it =  m_styleList.begin(); it != m_styleList.end(); counter++, it++ )
+    for( it =  m_styleList.begin(); it != end; counter++, ++it )
     {
         if ( (*it).styleName == styleName )
         {
