@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 
@@ -40,15 +40,16 @@
 #include <kaction.h>
 #include <kapplication.h>
 #include <kdebug.h>
-#include <koGlobal.h>
+#include <KoGlobal.h>
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kstdaction.h>
 #include <kstandarddirs.h>
 #include <kurl.h>
-#include <koxmlwriter.h>
-#include <koStoreDevice.h>
+#include <KoXmlWriter.h>
+#include <KoStoreDevice.h>
 #include <ktempfile.h>
+#include <KoMainWindow.h>
 
 #include <kformulacontainer.h>
 #include <kformuladocument.h>
@@ -96,7 +97,6 @@ bool KFormulaDoc::saveOasis( KoStore* store, KoXmlWriter* manifestWriter )
     KTempFile contentTmpFile;
     contentTmpFile.setAutoDelete( true );
     QFile* tmpFile = contentTmpFile.file();
-    KoXmlWriter contentTmpWriter( tmpFile, 1 );
 
     //todo save content
     QTextStream stream(tmpFile);
@@ -174,12 +174,22 @@ void KFormulaDoc::documentRestored()
 }
 
 
-bool KFormulaDoc::initDoc(InitDocFlags flags, QWidget* parentWidget)
+bool KFormulaDoc::initDoc(InitDocFlags /*flags*/, QWidget* /*parentWidget*/)
 {
     // If nothing is loaded, do initialize here
     return TRUE;
 }
 
+void KFormulaDoc::showStartUpWidget(KoMainWindow* parent, bool /*alwaysShow*/)
+{
+    parent->setRootDocument( this );
+}
+
+bool KFormulaDoc::showEmbedInitDialog(QWidget* /*parent*/)
+{
+  return true;
+}
+
 void KFormulaDoc::paintContent(QPainter& painter, const QRect& rect, bool transparent, double zoomX, double zoomY)
 {
     // ####### handle transparency and zoom
